"use strict";

var _vue = require("vue");

var _utils = require("./utils");

var _templateObject, _templateObject2, _templateObject3, _templateObject4;

function _taggedTemplateLiteral(strings, raw) { if (!raw) { raw = strings.slice(0); } return Object.freeze(Object.defineProperties(strings, { raw: { value: Object.freeze(raw) } })); }

var styled;

var stripLineBreaks = function stripLineBreaks(str) {
  return str.split('\n').map(function (l) {
    return l.trim();
  }).join('');
};

describe('css features', function () {
  beforeEach(function () {
    styled = (0, _utils.resetStyled)();
  });
  it('should add vendor prefixes in the right order', function () {
    var Comp = styled.div(_templateObject || (_templateObject = _taggedTemplateLiteral(["\n      transition: opacity 0.3s;\n    "])));
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)('.a {-webkit-transition: opacity 0.3s;transition: opacity 0.3s;}');
  });
  it('should add vendor prefixes for display', function () {
    var Comp = styled.div(_templateObject2 || (_templateObject2 = _taggedTemplateLiteral(["\n      display: flex;\n      flex-direction: column;\n      align-items: center;\n    "])));
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)(stripLineBreaks("\n      .a {\n        display: -webkit-box;\n        display: -webkit-flex;\n        display: -ms-flexbox;\n        display: flex;\n        -webkit-flex-direction: column;\n        -ms-flex-direction: column;\n        flex-direction: column;\n        -webkit-align-items: center;\n        -webkit-box-align: center;\n        -ms-flex-align: center;\n        align-items: center;\n      }\n    "));
  });
  it('should handle CSS calc()', function () {
    var Comp = styled.div(_templateObject3 || (_templateObject3 = _taggedTemplateLiteral(["\n      margin-bottom: calc(15px - 0.5rem) !important;\n    "])));
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)('.a {margin-bottom: calc(15px - 0.5rem) !important;}');
  });
  it('should pass through custom properties', function () {
    var Comp = styled.div(_templateObject4 || (_templateObject4 = _taggedTemplateLiteral(["\n      --custom-prop: some-val;\n    "])));
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)('.a {--custom-prop: some-val;}');
  });
});