"use strict";

var _vue = require("vue");

var _expect = _interopRequireDefault(require("expect"));

var _utils = require("./utils");

var _templateObject, _templateObject2, _templateObject3, _templateObject4, _templateObject5, _templateObject6, _templateObject7, _templateObject8, _templateObject9, _templateObject10, _templateObject11, _templateObject12, _templateObject13, _templateObject14;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _taggedTemplateLiteral(strings, raw) { if (!raw) { raw = strings.slice(0); } return Object.freeze(Object.defineProperties(strings, { raw: { value: Object.freeze(raw) } })); }

var styled;
describe('extending components', function () {
  beforeEach(function () {
    styled = (0, _utils.resetStyled)();
  });
  it('should generate a single class if only parent has styles', function () {
    var Parent = styled.div(_templateObject || (_templateObject = _taggedTemplateLiteral(["color: blue;"])));
    var Child = styled(Parent)(_templateObject2 || (_templateObject2 = _taggedTemplateLiteral([""])));
    var p = (0, _vue.createApp)(Parent).mount('body');
    var c = (0, _vue.createApp)(Child).mount('body');
    (0, _utils.expectCSSMatches)('.a {color: blue;}');
  });
  it('should generate a single class if only child has styles', function () {
    var Parent = styled.div(_templateObject3 || (_templateObject3 = _taggedTemplateLiteral(["color: blue;"])));
    var Child = styled(Parent)(_templateObject4 || (_templateObject4 = _taggedTemplateLiteral([""])));
    var p = (0, _vue.createApp)(Parent).mount('body');
    var c = (0, _vue.createApp)(Child).mount('body');
    (0, _utils.expectCSSMatches)('.a {color: blue;}');
  });
  it('should generate a new class for the child with the added rules', function () {
    var Parent = styled.div(_templateObject5 || (_templateObject5 = _taggedTemplateLiteral(["background-color: blue;"])));
    var Child = styled(Parent)(_templateObject6 || (_templateObject6 = _taggedTemplateLiteral(["color: red;"])));
    var p = (0, _vue.createApp)(Parent).mount('body');
    var c = (0, _vue.createApp)(Child).mount('body');
    (0, _utils.expectCSSMatches)('.a {background-color: blue;} .b {color: red;}');
  });
  it('should generate different classes for both parent and child', function () {
    var Parent = styled.div(_templateObject7 || (_templateObject7 = _taggedTemplateLiteral(["color: blue;"])));
    var Child = styled(Parent)(_templateObject8 || (_templateObject8 = _taggedTemplateLiteral(["color: red;"])));
    var p = (0, _vue.createApp)(Parent).mount('body');
    var c = (0, _vue.createApp)(Child).mount('body');
    (0, _utils.expectCSSMatches)('.a {color: blue;} .b {color: red;}');
  });
  it('should keep nested rules to the child', function () {
    var Parent = styled.div(_templateObject9 || (_templateObject9 = _taggedTemplateLiteral(["\n      color: blue;\n      > h1 { font-size: 4rem; }\n    "])));
    var Child = styled(Parent)(_templateObject10 || (_templateObject10 = _taggedTemplateLiteral(["color: red;"])));
    var p = (0, _vue.createApp)(Parent).mount('body');
    var c = (0, _vue.createApp)(Child).mount('body');
    (0, _utils.expectCSSMatches)('.a {color: blue;}.a > h1 {font-size: 4rem;} .b {color: red;}');
  });
  it('should keep default props from parent', function () {
    var parentProps = {
      color: {
        type: String,
        "default": 'red'
      }
    };
    var Parent = styled('div', parentProps)(_templateObject11 || (_templateObject11 = _taggedTemplateLiteral(["\n      color: ", ";\n    "])), function (props) {
      return props.color;
    });
    var Child = styled(Parent)(_templateObject12 || (_templateObject12 = _taggedTemplateLiteral(["background-color: green;"])));
    var p = (0, _vue.createApp)(Parent).mount('body');
    var c = (0, _vue.createApp)(Child).mount('body');
    (0, _utils.expectCSSMatches)("\n      .a {color: red;}\n      .b {background-color: green;}\n    ");
  });
  it('should keep prop types from parent', function () {
    var parentProps = {
      color: {
        type: String
      }
    };
    var Parent = styled.div(_templateObject13 || (_templateObject13 = _taggedTemplateLiteral(["\n      color: ", ";\n    "])), function (props) {
      return props.color;
    });
    var Child = styled(Parent)(_templateObject14 || (_templateObject14 = _taggedTemplateLiteral(["background-color: green;"])));
    var c = (0, _vue.createApp)(Child).mount('body');
    var p = (0, _vue.createApp)(Parent).mount('body');
    (0, _expect["default"])(c.$props).toEqual(p.$props);
  });
});