"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

var _vue = _interopRequireWildcard(require("vue"));

var _utils = require("./utils");

var _templateObject, _templateObject2, _templateObject3, _templateObject4, _templateObject5, _templateObject6, _templateObject7, _templateObject8, _templateObject9, _templateObject10, _templateObject11, _templateObject12, _templateObject13;

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _taggedTemplateLiteral(strings, raw) { if (!raw) { raw = strings.slice(0); } return Object.freeze(Object.defineProperties(strings, { raw: { value: Object.freeze(raw) } })); }

var styled;
describe('with styles', function () {
  beforeEach(function () {
    styled = (0, _utils.resetStyled)();
  });
  it('should append a style', function () {
    var rule = 'color: blue;';
    var Comp = styled.div(_templateObject || (_templateObject = _taggedTemplateLiteral(["\n        ", "\n      "])), rule);
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)('.a {color: blue;}');
  });
  it('should append multiple styles', function () {
    var rule1 = 'color: blue;';
    var rule2 = 'background: red;';
    var Comp = styled.div(_templateObject2 || (_templateObject2 = _taggedTemplateLiteral(["\n        ", "\n        ", "\n      "])), rule1, rule2);
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)('.a {color: blue;background: red;}');
  });
  it('should handle inline style objects', function () {
    var rule1 = {
      backgroundColor: 'blue'
    };
    var Comp = styled.div(_templateObject3 || (_templateObject3 = _taggedTemplateLiteral(["\n        ", "\n      "])), rule1);
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)('.a {background-color: blue;}');
  });
  it('should handle inline style objects with media queries', function () {
    var rule1 = {
      backgroundColor: 'blue',
      '@media screen and (min-width: 250px)': {
        backgroundColor: 'red'
      }
    };
    var Comp = styled.div(_templateObject4 || (_templateObject4 = _taggedTemplateLiteral(["\n        ", "\n      "])), rule1);
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)('.a {background-color: blue;}@media screen and (min-width: 250px) {.a {background-color: red;}}');
  });
  it('should handle inline style objects with pseudo selectors', function () {
    var rule1 = {
      backgroundColor: 'blue',
      '&:hover': {
        textDecoration: 'underline'
      }
    };
    var Comp = styled.div(_templateObject5 || (_templateObject5 = _taggedTemplateLiteral(["\n      ", "\n    "])), rule1);
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)('.a {background-color: blue;}.a:hover {-webkit-text-decoration: underline;text-decoration: underline;}');
  });
  it('should handle inline style objects with pseudo selectors', function () {
    var rule1 = {
      backgroundColor: 'blue',
      '&:hover': {
        textDecoration: 'underline'
      }
    };
    var Comp = styled.div(_templateObject6 || (_templateObject6 = _taggedTemplateLiteral(["\n      ", "\n    "])), rule1);
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)('.a {background-color: blue;}.a:hover {-webkit-text-decoration: underline;text-decoration: underline;}');
  });
  it('should handle inline style objects with nesting', function () {
    var rule1 = {
      backgroundColor: 'blue',
      '> h1': {
        color: 'white'
      }
    };
    var Comp = styled.div(_templateObject7 || (_templateObject7 = _taggedTemplateLiteral(["\n      ", "\n    "])), rule1);
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)('.a {background-color: blue;}.a > h1 {color: white;}');
  });
  it('should handle inline style objects with contextual selectors', function () {
    var rule1 = {
      backgroundColor: 'blue',
      'html.something &': {
        color: 'white'
      }
    };
    var Comp = styled.div(_templateObject8 || (_templateObject8 = _taggedTemplateLiteral(["\n      ", "\n    "])), rule1);
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)('.a {background-color: blue;}html.something .a {color: white;}');
  });
  it('should inject styles of multiple components', function () {
    var firstRule = 'background: blue;';
    var secondRule = 'background: red;';
    var FirstComp = styled.div(_templateObject9 || (_templateObject9 = _taggedTemplateLiteral(["\n        ", "\n      "])), firstRule);
    var SecondComp = styled.div(_templateObject10 || (_templateObject10 = _taggedTemplateLiteral(["\n        ", "\n      "])), secondRule);
    var vm1 = (0, _vue.createApp)(FirstComp).mount('body');
    var vm2 = (0, _vue.createApp)(SecondComp).mount('body');
    (0, _utils.expectCSSMatches)('.a {background: blue;} .b {background: red;}');
  });
  it('should inject styles of multiple components based on creation, not rendering order', function () {
    var firstRule = 'content: "first rule";';
    var secondRule = 'content: "second rule";';
    var FirstComp = styled.div(_templateObject11 || (_templateObject11 = _taggedTemplateLiteral(["\n        ", "\n      "])), firstRule);
    var SecondComp = styled.div(_templateObject12 || (_templateObject12 = _taggedTemplateLiteral(["\n        ", "\n      "])), secondRule);
    var vm2 = (0, _vue.createApp)(SecondComp).mount('body');
    var vm1 = (0, _vue.createApp)(FirstComp).mount('body');
    (0, _utils.expectCSSMatches)("\n        .b {content: \"first rule\";}\n        .a {content: \"second rule\";}\n      ");
  });
  it('should strip a JS-style (invalid) comment in the styles', function () {
    var comment = '// This is an invalid comment';
    var rule = 'color: blue;';
    var Comp = styled.div(_templateObject13 || (_templateObject13 = _taggedTemplateLiteral(["\n        ", "\n        ", "\n      "])), comment, rule);
    var vm = (0, _vue.createApp)(Comp).mount('body');
    (0, _utils.expectCSSMatches)("\n        .a {color: blue;}\n      ");
  });
});